/*
    lib/comedi.c
    comedi library routines

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>

#include <libinternal.h>

int __comedi_init=0;

void initialize(void)
{
	char *s;
	
	__comedi_init=1;

	if( (s=getenv("COMEDILIB_LOGLEVEL")) ){
		sscanf(s,"%d",&__comedi_loglevel);
		fprintf(stderr,"setting loglevel to %d\n",__comedi_loglevel);
	}
}
  
comedi_t *comedi_open(char *fn)
{
	comedi_t *it;

	if(!__comedi_init)
		initialize();

	if(!(it=malloc(sizeof(comedi_t))))
		goto cleanup;
	memset(it,0,sizeof(comedi_t));

	if((it->fd=open(fn,O_RDWR))<0){
		libc_error();
		goto cleanup;
	}

	if(ioctl_devinfo(it->fd,&it->devinfo)<0)
		goto cleanup;

	it->n_subdevices=it->devinfo.n_subdevs;

	get_subdevices(it);

	it->magic=COMEDILIB_MAGIC;

	return it;
cleanup:
	if(it)
		free(it);

	return NULL;
}

#if 0
/* this is an example of how we do versioned symbols */
__asm__(".symver comedi_open_0,comedi_open@");
#endif

void comedi_close(comedi_t *it)
{
	it->magic=0;

	/* XXX should free all memory */

	close(it->fd);
}

int comedi_fileno(comedi_t *it)
{
	if(!it)
		return -1;

	return it->fd;
}

int comedi_trigger(comedi_t *it,comedi_trig *t)
{
	if(!it || !t)
		return -1;

	return ioctl_trigger(it->fd,t);
}

int comedi_get_n_subdevices(comedi_t *it)
{
	if(!valid_dev(it))
		return -1;

	return it->n_subdevices;
}

int comedi_get_version_code(comedi_t *it)
{
	if(!valid_dev(it))
		return -1;

	return it->devinfo.version_code;
}

char *comedi_get_driver_name(comedi_t *it)
{
	if(!valid_dev(it))
		return NULL;

	return it->devinfo.driver_name;
}

char *comedi_get_board_name(comedi_t *it)
{
	if(!valid_dev(it))
		return NULL;

	return it->devinfo.board_name;
}

int comedi_get_subdevice_type(comedi_t *it,unsigned int subd)
{
	if(!valid_dev(it))
		return -1;

	return it->subdevices[subd].type;
}

#define COMEDI_SUBD_MAX COMEDI_SUBD_PROC

int comedi_find_subdevice_by_type(comedi_t *it,int type,unsigned int subd)
{
	if(!valid_subd(it,subd))
		return -1;

	if(type>=COMEDI_SUBD_MAX)
		return -1;
	
	for(;subd<it->n_subdevices;subd++){
		if(it->subdevices[subd].type==type)
			return subd;
	}
	return -1;
}


int comedi_get_n_channels(comedi_t *it,unsigned int subd)
{
	if(!valid_subd(it,subd))
		return -1;

	return it->subdevices[subd].n_chan;
}


/* */

lsampl_t comedi_get_maxdata(comedi_t *it,unsigned int subdevice,unsigned int chan)
{
	if(!valid_chan(it,subdevice,chan))
		return 0;

	if(it->subdevices[subdevice].maxdata_list)
		return it->subdevices[subdevice].maxdata_list[chan];

	return it->subdevices[subdevice].maxdata;
}

int comedi_get_rangetype(comedi_t *it,unsigned int subdevice,unsigned int chan)
{
	if(!valid_chan(it,subdevice,chan))
		return -1;

	if(it->subdevices[subdevice].range_type_list)
		return it->subdevices[subdevice].range_type_list[chan];

	return it->subdevices[subdevice].range_type;
}


comedi_range * comedi_get_range(comedi_t *it,unsigned int subdevice,unsigned int chan,unsigned int range)
{
	int range_type;

	if(!valid_chan(it,subdevice,chan))
		return NULL;

	range_type=comedi_get_rangetype(it,subdevice,chan);

	if(range>=RANGE_LENGTH(range_type))
		return NULL;

	if(it->subdevices[subdevice].rangeinfo_list)
		return it->subdevices[subdevice].rangeinfo_list[chan]+range;

	return it->subdevices[subdevice].rangeinfo+range;
}


/* */


int comedi_data_write(comedi_t *it,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref,lsampl_t data)
{
	comedi_trig cmd={
		mode:		0,
		flags:		0,
		n_chan:		1,
		n:		1,
		trigsrc:	0,
		trigvar:	0,
		trigvar1:	0,
	};
	sampl_t sdata=data;

	if(!valid_chan(it,subdev,chan))
		return -1;
	
	chan=CR_PACK(chan,range,aref);

	cmd.subdev=subdev;
	if(it->subdevices[subdev].subd_flags & SDF_LSAMPL){
		cmd.data=(sampl_t *)(&data);
	}else{
		cmd.data=&sdata;
	}
	cmd.chanlist=&chan;

	return ioctl_trigger(it->fd,&cmd);
}


int comedi_data_read(comedi_t *it,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref,lsampl_t *data)
{
	comedi_trig cmd={
		mode:		0,
		flags:		TRIG_WRITE,
		n_chan:		1,
		n:		1,
		trigsrc:	0,
		trigvar:	0,
		trigvar1:	0,
	};
	int ret;
	sampl_t sdata;

	if(!valid_chan(it,subdev,chan))
		return -1;
	
	chan=CR_PACK(chan,range,aref);
	
	cmd.subdev=subdev;
	cmd.chanlist=&chan;
	if(it->subdevices[subdev].subd_flags & SDF_LSAMPL){
		cmd.data=(sampl_t *)data;
	}else{
		cmd.data=&sdata;
	}

	ret=ioctl_trigger(it->fd,&cmd);
	if(ret<0)
		return ret;

	if(!(it->subdevices[subdev].subd_flags & SDF_LSAMPL)){
		*data=sdata;
	}

	return 0;
}


int comedi_dio_config(comedi_t *it,unsigned int subdev,unsigned int chan,unsigned int io)
{
	comedi_trig cmd={
		mode:		0,
		flags:		TRIG_CONFIG|TRIG_WRITE,
		n_chan:		1,
		n:		1,
		trigsrc:	0,
		trigvar:	0,
		trigvar1:	0,
	};
	sampl_t data=io;

	if(!valid_chan(it,subdev,chan))
		return -1;
	
	if(io!=COMEDI_INPUT || io!=COMEDI_OUTPUT)
		return -1;

	cmd.subdev=subdev;
	cmd.chanlist=&chan;
	cmd.data=&data;

	return ioctl_trigger(it->fd,&cmd);
}

